<?php

class Controller_Api_V0_SendGrades extends Controller_Handler_Api{

    /**
     * @api {put} api/v0/sendGrades set grades for student in discipline by submodule
     * @apiName Set grade
     * @apiGroup Grades
     * @apiVersion 0.1.2
     * @apiParam {String} token Api key
     * @apiDescription This method accepts json body
     *    with list of submodules and grades to add or update
     * @apiExample {curl} Example usage:
     * curl --header "Content-Type: application/json" \
     * --request POST \
     * --data '{
     *      "discipline": 3726,
     *      "recordbook": 33434,
     *      "submodules": [
     *           {
     *               "35712": 2.6,
     *               "35713": 2.1
     *           }
     *       ]
     * }' \
     *   http://grade/~dev_rating/api/v0/sendGrades?token=osw839hfgh9a23hgfh92hasff232f2oasf
     */
    public function action_put_index()
    {
        $res = [];

        try {
            $data = json_decode($this->request->body());

            //$discipline = (int)$data->discipline;
            //$recordbook = (int)$data->recordbook;
            $subject = $data->subject;
            $nick = $data->nick;
            $service = $data->service;
            $semester = $data->semester;
                //getByNickSubject
            $result = Model_RecordBook::getByNickSubject($nick, $subject, $semester, $service);
            $recordbook = $result[0]["RecordBookID"];
            $discipline = $result[0]["DisciplineID"];

            foreach ($data->submodules as $submoduleData) {
                #$res[] = $this->processStudyPlan($studyPlanData, $year, $semesterID, $facultyID);
                $d = Model_Discipline::load($discipline);
                if ($d->isTaskOpen($recordbook, (int)$submoduleData->number)) {
                    $submoduleID = Model_Map::getSubmoduleByNumber($discipline, (int)$submoduleData->number);
                    $error = Model_Rating::SetRate(
                        $this->user->TeacherID,
                        $recordbook,
                        $discipline,
                        $submoduleID,
                        (int)$submoduleData->value
                    );
                }
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }
}
