<?php

class Controller_Api_V0_Teacher extends Controller_Handler_Api {
    private function normalizeTeacherData($teacherData) {
        if (!isset($teacherData->id) && isset($teacherData->externalID)) {
            $teacherData->id = Model_Teacher::withExternalID($teacherData->externalID);
        }

        if (!isset($teacherData->id) || ($teacherData->id == -1)) {
            $foundList = Model_Teachers::search([$teacherData->firstName, $teacherData->secondName, $teacherData->lastName], $teacherData->facultyID);
            if (count($foundList) == 1) {
                $teacherData->id = $foundList[0]['ID'];
            } elseif (count($foundList) > 1) {
                throw new InvalidArgumentException('Duplicate teachers');
            }
        }

        if (!isset($teacherData->departmentID)) {
            if (isset($teacherData->departmentExternalID)) {
                $depid = $this->getDepIDByExternalID($teacherData->departmentExternalID);
                // !!! TODO: implement department synchronization
//                if ( $depid === NULL ) {
//                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка коде справочника подразделения.') );
//                }
                $teacherData->departmentID = $depid;
            } elseif (isset($teacherData->departmentName)) {
                $depid = $this->getDepIDByName($teacherData->departmentName);
//                if ( $depid === NULL ) {
//                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка в имени подразделения.') );
//                }
                $teacherData->departmentID = $depid;
            } else {
                throw new InvalidArgumentException('Teacher has no department');
            }
        }

        if (!isset($teacherData->jobPositionID) && isset($teacherData->jobPositionName)) {
            $teacherData->jobPositionID = Model_Faculties::getJobPositionIdByName($teacherData->jobPositionName);
            if (is_null($teacherData->jobPositionID)) {
                $teacherData->jobPositionID = Model_Faculties::createJobPosition($teacherData->jobPositionName);
            }
        } elseif (!isset($teacherData->jobPositionName)) {
            throw new InvalidArgumentException('Teacher has no job position');
        }

        switch ($teacherData->status) {
            case 'Работает':
                $teacherData->status = true;
                break;

            case 'Уволен':
                $teacherData->status = false;
                break;

            default:
                throw new InvalidArgumentException('Teacher has bad status');
        }

        return $teacherData;
    }

    private function createTeacher($teacherData) {
        $teacher = Model_Teacher::make()
            ->faculty($teacherData->facultyID)
            ->jobPosition($teacherData->jobPositionID)
            ->firstName($teacherData->firstName)
            ->secondName($teacherData->secondName)
            ->lastName($teacherData->lastName);
        if (isset($teacherData->departmentID)) {
            $teacher->department($teacherData->departmentID);
        }
        if (isset($teacherData->externalID)) {
            $teacher = $teacher->externalID($teacherData->externalID);
        }
        if (isset($teacherData->inila)) {
            $teacher = $teacher->inila($teacherData->inila);
        }
        $teacher = $teacher->create();
        if (isset($teacher->ActivationCode)) {
            return [(int)$teacher->ID, $teacher->ActivationCode];
        } else {
            return (int)$teacher->ID;
        }
    }

    private function updateTeacher($teacherData) {
        $teacher = Model_Teacher::with($teacherData->id);
        $teacher->changeInfo($teacherData->externalID, $teacherData->inila,
            $teacherData->lastName, $teacherData->firstName, $teacherData->secondName,
            $teacherData->jobPositionID, $teacherData->departmentID, $teacherData->status
        );

        return (int)$teacher->ID;
    }

     /**
     * @api {put} api/v0/teacher/WithExternalID Add or update teacher(s) with particular 1C ID
     * @apiName Update teacher with external id
     * @apiGroup Teacher
     * @apiVersion 0.1.1
     * @apiParam {String} token Api-токен
     * @apiDescription This method accepts json body
     *    with list of teachers to add or update.
     * @apiExample {curl} Example usage:
     * curl --header "Content-Type: application/json" \
     * --request POST \
     * --data '{
     *  "faculty": "000001387",
     *  "teachers": [
     *  {
     *  "status" : "Работает", # Уволен,Работает
     *  "firstName" : "Иван", # Имя преподавателя
     *  "lastName" : "Иванова", # Фамилия преподавателя
     *  "inila" : "LSJF9QAH0ASHFWAQFASF22FSAF", # Хэш СНИЛС преподавателя (для авторизации через OpenID.sfedu.ru)
     *  "externalID" : "4300000000", # Код справочника для этого преподавателя из 1С:Университет
     *  "secondName" : "Иванович", # Отчество преподавателя
     *  "departmentExternalID" : "000001387", # Код справочника из 1С:Университет
     *  "jobPositionName" : "Старший преподаватель" # Название должности.
     *  }
     *  ]
     *  }' \
     *   http://grade/~dev_rating/api/v0/teacher?token=osw839hfgh9a23hgfh92hasff232f2oasf
     */
    public function action_put_index() {
        $res = [];

        try {
            $data = json_decode($this->request->body());
            $facultyID = Model_Faculties::getIdByExternalID($data->faculty);

            foreach ($data->teachers as $teacherData) {
                $teacherData->facultyID = $facultyID;

                try {
                    $teacherData = $this->normalizeTeacherData($teacherData);

                    if (isset($teacherData->id) && $teacherData->id > 0) {
                        $res[] = $this->updateTeacher($teacherData);
                    } else {
                        $res[] = $this->createTeacher($teacherData);
                    }
                } catch (Exception $e) {
                    Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                        '{0}' => 'SYNC_TEACHERS',
                        '{1}' => $e->getMessage(),
                        '{2}' => json_encode($teacherData),
                    ));
                }
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }
}
